package shared_workspace

import (
	"encoding/json"

	"cloud.o-forge.io/core/oc-lib/models/peer"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	w "cloud.o-forge.io/core/oc-lib/models/workflow"
	"cloud.o-forge.io/core/oc-lib/models/workspace"
	"cloud.o-forge.io/core/oc-lib/models/workspace/shared/rules/rule"
	"cloud.o-forge.io/core/oc-lib/tools"
	"github.com/google/uuid"
)

type SharedWorkspace struct {
	utils.AbstractObject
	IsSent      bool                   `json:"is_sent" bson:"-"`
	CreatorID   string                 `json:"peer_id,omitempty" bson:"peer_id,omitempty" validate:"required"`
	Version     string                 `json:"version,omitempty" bson:"version,omitempty"`
	Description string                 `json:"description,omitempty" bson:"description,omitempty" validate:"required"`
	Attributes  map[string]interface{} `json:"attributes,omitempty" bson:"attributes,omitempty"`
	Workspaces  []string               `json:"workspaces,omitempty" bson:"workspaces,omitempty"`
	Workflows   []string               `json:"workflows,omitempty" bson:"workflows,omitempty"`
	Peers       []string               `json:"peers,omitempty" bson:"peers,omitempty"`
	Rules       []string               `json:"rules,omitempty" bson:"rules,omitempty"`

	SharedRules      []*rule.Rule           `json:"shared_rules,omitempty" bson:"-"`
	SharedWorkspaces []*workspace.Workspace `json:"shared_workspaces,omitempty" bson:"-"`
	SharedWorkflows  []*w.Workflow          `json:"shared_workflows,omitempty" bson:"-"`
	SharedPeers      []*peer.Peer           `json:"shared_peers,omitempty" bson:"-"`
}

func (ao *SharedWorkspace) GetID() string {
	return ao.UUID
}

func (r *SharedWorkspace) GenerateID() {
	if r.UUID == "" {
		r.UUID = uuid.New().String()
	}
}

func (d *SharedWorkspace) GetName() string {
	return d.Name
}

func (d *SharedWorkspace) GetAccessor(caller *tools.HTTPCaller) utils.Accessor {
	data := New()
	data.Init(utils.SHARED_WORKSPACE, caller)
	return data
}

func (dma *SharedWorkspace) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *SharedWorkspace) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}
