package shared_workspace

import (
	"fmt"
	"slices"

	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/dbs/mongo"
	"cloud.o-forge.io/core/oc-lib/models/peer"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	w "cloud.o-forge.io/core/oc-lib/models/workflow"
	"cloud.o-forge.io/core/oc-lib/models/workspace"
	"cloud.o-forge.io/core/oc-lib/models/workspace/shared/rules/rule"
	"cloud.o-forge.io/core/oc-lib/static"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type sharedWorkspaceMongoAccessor struct {
	utils.AbstractAccessor
}

func New() *sharedWorkspaceMongoAccessor {
	return &sharedWorkspaceMongoAccessor{}
}

func (wfa *sharedWorkspaceMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	set, code, _ := wfa.LoadOne(id)
	if code == 200 {
		wfa.deleteToPeer(set.(*SharedWorkspace))
	}
	wfa.sharedWorkflow(&SharedWorkspace{}, id)
	wfa.sharedWorkspace(&SharedWorkspace{}, id)
	return wfa.GenericDeleteOne(id, wfa)
}

func (wfa *sharedWorkspaceMongoAccessor) sharedWorkspace(shared *SharedWorkspace, id string) {
	eldest, code, _ := wfa.LoadOne(id)
	accessor := (&workspace.Workspace{}).GetAccessor(nil)
	if code == 200 {
		eld := eldest.(*SharedWorkspace)
		if eld.Workspaces != nil {
			for _, v := range eld.Workspaces {
				accessor.UpdateOne(&workspace.Workspace{Shared: shared.UUID}, v)
				if wfa.Caller != nil || wfa.Caller.URLS == nil || wfa.Caller.URLS[utils.WORKSPACE.String()] == nil {
					continue
				}
				paccess := (&peer.Peer{})
				for _, p := range shared.Peers {
					b, err := paccess.LaunchPeerExecution(p, v, utils.WORKSPACE, tools.DELETE, nil, wfa.Caller)
					if err != nil && b == nil {
						wfa.Logger.Error().Msg("Could not send to peer " + p + ". Error: " + err.Error())
					}
				}
			}
		}
	}
	if shared.Workspaces != nil {
		for _, v := range shared.Workspaces {
			workspace, code, _ := accessor.UpdateOne(&workspace.Workspace{Shared: shared.UUID}, v)
			if wfa.Caller != nil || wfa.Caller.URLS == nil || wfa.Caller.URLS[utils.WORKSPACE.String()] == nil {
				continue
			}
			for _, p := range shared.Peers {
				if code != 200 {
					continue
				}
				paccess := (&peer.Peer{})
				s := workspace.Serialize()
				s["name"] = fmt.Sprintf("%v", s["name"]) + "_" + p
				b, err := paccess.LaunchPeerExecution(p, v, utils.WORKSPACE, tools.POST, s, wfa.Caller)
				if err != nil && b == nil {
					wfa.Logger.Error().Msg("Could not send to peer " + p + ". Error: " + err.Error())
				}
			}
		}
	}
}

func (wfa *sharedWorkspaceMongoAccessor) sharedWorkflow(shared *SharedWorkspace, id string) {
	accessor := (&w.Workflow{}).GetAccessor(nil)
	eldest, code, _ := wfa.LoadOne(id)
	if code == 200 {
		eld := eldest.(*SharedWorkspace)
		if eld.Workflows != nil {
			for _, v := range eld.Workflows {
				data, code, _ := accessor.LoadOne(v)
				if code == 200 {
					s := data.(*w.Workflow)
					new := []string{}
					for _, id2 := range s.Shared {
						if id2 != id {
							new = append(new, id2)
						}
					}
					n := &w.Workflow{}
					n.Shared = new
					accessor.UpdateOne(n, v)
					if wfa.Caller != nil || wfa.Caller.URLS == nil || wfa.Caller.URLS[utils.WORKFLOW.String()] == nil {
						continue
					}
					paccess := (&peer.Peer{})
					for _, p := range shared.Peers {
						b, err := paccess.LaunchPeerExecution(p, v, utils.WORKFLOW, tools.DELETE, nil, wfa.Caller)
						if err != nil && b == nil {
							wfa.Logger.Error().Msg("Could not send to peer " + p + ". Error: " + err.Error())
						}
					}
				}
			}
		}
	}
	if shared.Workflows != nil {
		for _, v := range shared.Workflows {
			data, code, _ := accessor.LoadOne(v)
			if code == 200 {
				s := data.(*w.Workflow)
				if !slices.Contains(s.Shared, id) {
					s.Shared = append(s.Shared, id)
					workflow, code, _ := accessor.UpdateOne(s, v)
					if wfa.Caller != nil || wfa.Caller.URLS == nil || wfa.Caller.URLS[utils.WORKFLOW.String()] == nil {
						continue
					}
					paccess := (&peer.Peer{})
					for _, p := range shared.Peers {
						if code == 200 {
							s := workflow.Serialize()
							s["name"] = fmt.Sprintf("%v", s["name"]) + "_" + p
							b, err := paccess.LaunchPeerExecution(p, shared.UUID, utils.WORKFLOW, tools.POST, s, wfa.Caller)
							if err != nil && b == nil {
								wfa.Logger.Error().Msg("Could not send to peer " + p + ". Error: " + err.Error())
							}
						}
					}
				}
			}
		}
	}
}

func (wfa *sharedWorkspaceMongoAccessor) deleteToPeer(shared *SharedWorkspace) {
	if wfa.Caller == nil || wfa.Caller.URLS == nil || wfa.Caller.URLS[utils.SHARED_WORKSPACE.String()] == nil {
		return
	}
	paccess := (&peer.Peer{})
	for _, v := range shared.Peers {
		if (&peer.Peer{AbstractObject: utils.AbstractObject{UUID: v}}).IsMySelf() {
			continue
		}
		b, err := paccess.LaunchPeerExecution(v, shared.UUID, utils.SHARED_WORKSPACE, tools.DELETE, nil, wfa.Caller)
		if err != nil && b == nil {
			wfa.Logger.Error().Msg("Could not send to peer " + v + ". Error: " + err.Error())
		}
	}
}

func (wfa *sharedWorkspaceMongoAccessor) sendToPeer(shared *SharedWorkspace) {
	if wfa.Caller == nil || wfa.Caller.URLS == nil || wfa.Caller.URLS[utils.SHARED_WORKSPACE.String()] == nil {
		return
	}

	paccess := (&peer.Peer{})
	for _, v := range shared.Peers {
		if (&peer.Peer{AbstractObject: utils.AbstractObject{UUID: v}}).IsMySelf() || shared.IsSent {
			continue
		}
		shared.IsSent = true
		b, err := paccess.LaunchPeerExecution(v, v, utils.SHARED_WORKSPACE, tools.POST, shared.Serialize(), wfa.Caller)
		if err != nil && b == nil {
			wfa.Logger.Error().Msg("Could not send to peer " + v + ". Error: " + err.Error())
		}
	}
}

func (wfa *sharedWorkspaceMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	res, code, err := wfa.GenericUpdateOne(set.(*SharedWorkspace), id, wfa, &SharedWorkspace{})
	fmt.Println("UPDATE SHARED", res.Serialize())
	wfa.deleteToPeer(res.(*SharedWorkspace))
	wfa.sharedWorkflow(res.(*SharedWorkspace), id)
	wfa.sharedWorkspace(res.(*SharedWorkspace), id)
	wfa.sendToPeer(res.(*SharedWorkspace))
	return res, code, err
}

func (wfa *sharedWorkspaceMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	id, _ := static.GetMyLocalJsonPeer()
	data.(*SharedWorkspace).CreatorID = id
	data.(*SharedWorkspace).Peers = append(data.(*SharedWorkspace).Peers, id)
	if data.(*SharedWorkspace).Workspaces == nil {
		data.(*SharedWorkspace).Workspaces = []string{}
	}
	if data.(*SharedWorkspace).Workflows == nil {
		data.(*SharedWorkspace).Workflows = []string{}
	}
	if data.(*SharedWorkspace).Rules == nil {
		data.(*SharedWorkspace).Rules = []string{}
	}

	d, code, err := wfa.GenericStoreOne(data.(*SharedWorkspace), wfa)

	if code == 200 {
		wfa.sharedWorkflow(d.(*SharedWorkspace), d.GetID())
		wfa.sharedWorkspace(d.(*SharedWorkspace), d.GetID())
		wfa.sendToPeer(d.(*SharedWorkspace))
	}
	return data, code, err
}

func (wfa *sharedWorkspaceMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return wfa.StoreOne(data)
}

func (wfa *sharedWorkspaceMongoAccessor) enrich(sharedWorkspace *SharedWorkspace) *SharedWorkspace {
	access := (&workspace.Workspace{}).GetAccessor(nil)
	res, code, _ := access.Search(&dbs.Filters{
		Or: map[string][]dbs.Filter{
			"abstractobject.id": {{Operator: dbs.IN.String(), Value: sharedWorkspace.Workspaces}},
		},
	}, "")
	if code == 200 {
		for _, r := range res {
			sharedWorkspace.SharedWorkspaces = append(sharedWorkspace.SharedWorkspaces, r.(*workspace.Workspace))
		}
	}
	access = (&w.Workflow{}).GetAccessor(nil)
	res, code, _ = access.Search(&dbs.Filters{
		Or: map[string][]dbs.Filter{
			"abstractobject.id": {{Operator: dbs.IN.String(), Value: sharedWorkspace.Workflows}},
		},
	}, "")
	if code == 200 {
		for _, r := range res {
			sharedWorkspace.SharedWorkflows = append(sharedWorkspace.SharedWorkflows, r.(*w.Workflow))
		}
	}
	access = (&peer.Peer{}).GetAccessor(nil)
	res, code, _ = access.Search(&dbs.Filters{
		Or: map[string][]dbs.Filter{
			"abstractobject.id": {{Operator: dbs.IN.String(), Value: sharedWorkspace.Peers}},
		},
	}, "")
	if code == 200 {
		for _, r := range res {
			sharedWorkspace.SharedPeers = append(sharedWorkspace.SharedPeers, r.(*peer.Peer))
		}
	}
	access = (&rule.Rule{}).GetAccessor(nil)
	res, code, _ = access.Search(&dbs.Filters{
		Or: map[string][]dbs.Filter{
			"abstractobject.id": {{Operator: dbs.IN.String(), Value: sharedWorkspace.Rules}},
		},
	}, "")
	if code == 200 {
		for _, r := range res {
			sharedWorkspace.SharedRules = append(sharedWorkspace.SharedRules, r.(*rule.Rule))
		}
	}
	return sharedWorkspace
}

func (wfa *sharedWorkspaceMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	var sharedWorkspace SharedWorkspace
	res_mongo, code, err := mongo.MONGOService.LoadOne(id, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve " + id + " from db. Error: " + err.Error())
		return nil, code, err
	}
	res_mongo.Decode(&sharedWorkspace)
	return wfa.enrich(&sharedWorkspace), 200, nil
}

func (wfa sharedWorkspaceMongoAccessor) LoadAll() ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	res_mongo, code, err := mongo.MONGOService.LoadAll(wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not retrieve any from db. Error: " + err.Error())
		return nil, code, err
	}
	var results []SharedWorkspace
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, wfa.enrich(&r))
	}
	return objs, 200, nil
}

func (wfa *sharedWorkspaceMongoAccessor) Search(filters *dbs.Filters, search string) ([]utils.ShallowDBObject, int, error) {
	objs := []utils.ShallowDBObject{}
	if (filters == nil || len(filters.And) == 0 || len(filters.Or) == 0) && search != "" {
		filters = &dbs.Filters{
			Or: map[string][]dbs.Filter{
				"abstractobject.name": {{Operator: dbs.LIKE.String(), Value: search}},
			},
		}
	}
	res_mongo, code, err := mongo.MONGOService.Search(filters, wfa.GetType())
	if err != nil {
		wfa.Logger.Error().Msg("Could not store to db. Error: " + err.Error())
		return nil, code, err
	}
	var results []SharedWorkspace
	if err = res_mongo.All(mongo.MngoCtx, &results); err != nil {
		return nil, 404, err
	}
	for _, r := range results {
		objs = append(objs, wfa.enrich(&r))
	}
	return objs, 200, nil
}
